@@ -10,18 +10,25 @@ genotypes <- function() {#(r, reg, genodata, mode, weights, fweights, beta.par,
 		Z <- subset.SnpMatrix(Z, measured.ids)
 		Z <- SnpMatrix2numeric(Z)
 	} else if (gtype == 3) {
-		if (getRversion() >= "3.2.0") {
-			invisible(capture.output(invisible(capture.output(Z <- seqminer::readVCFToMatrixByGene(genodata, geneFile, reg, annoType), type = 'output')), type = 'message'))
-		} else { Z <- seqminer::readVCFToMatrixByGene(genodata, geneFile, reg, annoType) }
-		if (is.null(Z[[1]])) return(list(m0 = 0, Z = NULL, w = NULL, pos = NULL))
-		Z <- t(Z[[1]])
-		if (!dim(Z)[2] == 0) Z <- Z[measured.ids, ]
+		if (length(annoType) == 1) { 
+			Z <- read.vcf(genodata, geneFile, reg, annoType)
+			if (is.null(Z[[1]])) return(list(m0 = 0, Z = NULL, w = NULL, pos = NULL))
+			Z <- t(Z[[1]])
+		} else { 
+			Z <- c()
+			for (anno in annoType) {
+				Z0 <- read.vcf(genodata, geneFile, reg, anno)
+				if (is.null(Z0[[1]])) next
+				Z <- cbind(Z, t(Z0[[1]]))
+			}
+		}
+		if (!dim(Z)[2] == 0) Z <- as.matrix(Z[measured.ids, ])
 	}
 	if (dim(Z)[2] == 0) return(list(m0 = 0, Z = NULL, w = NULL, pos = NULL))
 	w <- pos <- NULL
 	if (test == 'famFLM') { pos <- positions[reg]
 	} else if (!is.null(weights)) w <- weights[reg]
-	Z <- as.matrix(Z)
+
 	m0 <- dim(Z)[2]
 
 	Z[Z == -9] <- NA
@@ -31,26 +38,27 @@ genotypes <- function() {#(r, reg, genodata, mode, weights, fweights, beta.par,
 	if (any(is.na(MAF))) {
 		if (all(is.na(MAF))) return(list(m0 = m0, Z = NULL, w = w, pos = pos))
 		index <- !is.na(MAF)
-		Z <- Z[, index]
+		Z <- as.matrix(Z[, index])
 		MAF <- MAF[index]
 		if (test == 'famFLM') { pos <- as.vector(pos[index])
 		} else if (!is.null(weights)) w <- as.vector(w[index])
 	}
 
+	index <- sapply(1:dim(Z)[2], function(x) all(Z[, x] >= 0, na.rm = T) & all(Z[, x] <= 2, na.rm = T))
+	if (sum(index) == 0) return(list(m0 = m0, Z = 'not in range', w = NULL, pos = NULL))
+	Z <- as.matrix(Z[, index])
+	MAF <- MAF[index]
+
 	if (any(MAF > .5) & test %in% c('famBT', 'famSKAT')) {
 		if (flip.genotypes) {
 			v <- MAF > .5
 			Z[, v] <- abs(2 - Z[, v])
-			MAF <- 1 - MAF
+			MAF[v] <- 1 - MAF[v]
 		} else warning("Check whether minor allele homozygotes are coded as 2 at reg ", r, ", use 'flip.genotypes = TRUE' if needed")
 	}
 
 	if (mode != 'add') {
 		Z <- round(Z)
-		index <- sapply(1:dim(Z)[2], function(x) all(Z[, x] >= 0, na.rm = T) & all(Z[, x] <= 2, na.rm = T))
-		if (sum(index) == 0) return(list(m0 = m0, Z = 'not in range', w = NULL, pos = NULL))
-		Z <- Z[, index]
-		MAF <- MAF[index]
 		if (test == 'famFLM') { pos <- as.vector(pos[index])
 		} else if (!is.null(weights)) w <- as.vector(w[index])
 		if (mode == 'rec') Z[Z == 1] <- 0
@@ -101,11 +109,6 @@ genotypes <- function() {#(r, reg, genodata, mode, weights, fweights, beta.par,
 		}
 	}
 
-	if (test == 'famBT') {
-		if (!is.null(fweights)) w <- fweights(MAF)
-		return(list(m0 = m0, Z = Z, w = w, pos = pos))
-	}
-
 	if (omit.linear.dependent) {
 		# detect and eliminate linear-dependent genetic variants
 		dqr <- qr(Z) # QR-decomposition
@@ -120,11 +123,13 @@ genotypes <- function() {#(r, reg, genodata, mode, weights, fweights, beta.par,
 			w <- as.vector(w[index])
 		} else if (!is.null(fweights)) {
 			MAF <- MAF[index]
-			w <- fweights(MAF)
 		}
-	} else {
-		if (!is.null(fweights)) w <- fweights(MAF)
 	}
+	if (is.null(weights) & !is.null(fweights)) w <- fweights(MAF)
+	if (test == 'famBT') {
+		return(list(m0 = m0, Z = Z, w = w, pos = pos))
+	}
+
 	if (test == 'famFLM') {
 #		pos <- as.vector(pos[index])
 		if (any(is.na(pos))) {
@@ -155,3 +160,10 @@ flipper <- function(Z) {
 	}
 	return(Z)
 }
+
+read.vcf <- function(genodata, geneFile, reg, annoType) {
+	if (getRversion() >= "3.2.0") {
+		invisible(capture.output(invisible(capture.output(Z <- seqminer::readVCFToMatrixByGene(genodata, geneFile, reg, annoType), type = 'output')), type = 'message'))
+	} else { Z <- seqminer::readVCFToMatrixByGene(genodata, geneFile, reg, annoType) }
+	return(Z)
+}
\ No newline at end of file
